/*
    This file is part of Kismet

    Kismet is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Kismet is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Kismet; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __AP_MONITOR_PROTO_H__
#define __AP_MONITOR_PROTO_H__

// All elements are transmitted in network byte order!

// Monitor protocol version
#define MONITOR_PROTOVER        1

// Sequence window
#define MONITOR_SEQ_WINDOW      10

// Did we lose our way?
#define MONITOR_SENTINEL        0xDEADC0DE

#define MONTYPE_SERVERINFO      0
#define MONTYPE_ZONE            1
#define MONTYPE_AP              2
#define MONTYPE_INTERFACE       4
#define MONTYPE_MESSAGE         5

// Length of frames
#define MONLEN_HEADER           sizeof(monitor_header)

// For sizing the header
typedef struct monitor_header {
    uint32_t sentinel __attribute__ ((packed));
    uint8_t type __attribute__ ((packed));
    uint32_t len __attribute__ ((packed));
    uint64_t sequence_id __attribute__ ((packed));
};

// Monitor frames themselves
typedef struct monitor_frame {
    uint32_t sentinel __attribute__ ((packed));
    uint8_t type __attribute__ ((packed));
    uint32_t len __attribute__ ((packed));
    uint64_t sequence_id __attribute__ ((packed));
    union {
        // Server identification frame
        struct {
            uint8_t proto_ver;
            uint32_t start_time;
            uint16_t config_delay;
            uint16_t reconfig_delay;
            uint16_t deadidle_delay;
            uint16_t smite_delay;
            uint16_t smite_ignore;
            uint16_t sniffers_per;
            uint16_t rawtx_per;
            uint8_t base_bssid[6]; // not currently used
            uint8_t base_bssidmask[6]; // not currently used
            uint8_t base_ssid[32];

        } serverinfo __attribute__ ((packed));
        // Info about a specific zone
        struct {
            uint16_t id;
            uint8_t dirty;
            uint8_t nsniffer;
            uint8_t nrawtx;
            uint8_t nap;
            uint8_t nnodes;
            uint8_t ninterfaces;
            uint32_t nalerts;
            uint32_t nattacks;
        } zone __attribute__ ((packed));
        // Info about a specific AP in a zone
        struct {
            uint8_t name[32];
            uint16_t zoneid;
            uint16_t stationid;
            uint8_t configured;
            uint8_t active;
            uint8_t dead;
            uint32_t last_helo;
            uint8_t ninterfaces;
            uint8_t loadavg_major;
            uint8_t loadavg_minor;
        } ap __attribute__ ((packed));
        // Info about specific interfaces
        struct {
            uint8_t name[32];
            uint16_t stationid;
            uint8_t index;
            uint8_t mac[6];
            uint8_t innate_mac[6];
            uint16_t channel;
            uint32_t cf_bitmap;
            uint32_t last_mac_change;
            uint32_t nclients;
            uint64_t framecount;
            uint64_t errorcount;
            uint64_t txpacketcount;
            uint64_t txbytecount;
            uint64_t rxpacketcount;
            uint64_t rxbytecount;
        } interface __attribute__ ((packed));
        // A message frame from the standard message proto
        struct {
            uint32_t msgflags;
            uint32_t msglen;
            // Don't send entire msgtext block - rely on msglen
            uint8_t msgtext[1024];
        } message __attribute__ ((packed));
    } u;
};

#endif

