/*
    This file is part of Kismet

    Kismet is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Kismet is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Kismet; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "config.h"

/* According to POSIX 1003.1-2001 */
#include <sys/select.h>

/* According to earlier standards */
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

#include "messagebus.h"
#include "util.h"
#include "tcpserver.h"
#include "tcpheartserver.h"

// Global message bus
MessageBus mesbus;

int zero_count = 0;

int WatchdogMonitor(Timetracker::timer_event *evt, void *parm) {
    TcpHeartServer *hbs = (TcpHeartServer *) parm;

    time_t lastbeat = hbs->FetchLastBeat();

    /*
    printf("Time: %d last %d off %ld\n",
           (int) lastbeat, (int) time(0), (time(0) - lastbeat));
    */

    if (lastbeat == 0) {
        zero_count++;
        if (zero_count > 20) {
            fprintf(stderr, "FATAL:  No connection within 20 seconds.\n");
            exit(1);
        }
    }

    if (lastbeat != 0 && (time(0) - lastbeat) > 10) {
        fprintf(stderr, "FATAL:  No heartbeat in more than 10 seconds.\n");
        exit(1);
    }

    return 1;
}

int main(int argc, char *argv[]) {
    // Infrastructure bits
    TcpHeartServer hbs;
    StdoutMessageClient stdclient;
    Timetracker timetracker;
    fd_set master_readset, master_writeset;

    int max_fd = 0;
    int ret = 0;

    mesbus.RegisterClient(&stdclient, MSGFLAG_ALL);

    hbs.RegisterTimetracker(&timetracker);
    hbs.RegisterMessageBus(&mesbus);

    if (hbs.Setup(10, 31338) < 0) {
        exit(1);
    }

    timetracker.RegisterTimer(SERVER_TIMESLICES_SEC,
                              NULL, 1, &WatchdogMonitor, 
                              (void *) &hbs);

    while (1) {
        struct timeval tm;

        tm.tv_sec = 0;
        tm.tv_usec = 100000;

        FD_ZERO(&master_readset);
        FD_ZERO(&master_writeset);

        if ((max_fd = hbs.MergeSet(master_readset, master_writeset,
                                   max_fd, &master_readset, &master_writeset)) < 0) {
            fprintf(stderr, "FATAL: %s\n", hbs.FetchError());
            exit(1);
        }

        if (select(max_fd + 1, &master_readset, &master_writeset, NULL, &tm) < 0) {
            fprintf(stderr, "FATAL: select() failed: %s\n", strerror(errno));
            exit(1);
        }

        if ((ret = hbs.Poll(master_readset, master_writeset)) < 0) {
            fprintf(stderr, "FATAL: %s\n", hbs.FetchError());
            exit(1);
        } else if (ret == 0) {
            fprintf(stderr, "WARNING: %s\n", hbs.FetchError());
        }

        timetracker.Tick();
    }

}

