/*
    This file is part of Kismet

    Kismet is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Kismet is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Kismet; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __HOSTAPCONTROL_H__
#define __HOSTAPCONTROL_H__

#include "config.h"

#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <stdlib.h>
#include <pwd.h>

#include <errno.h>

#include <string>
#include <map>
#include <vector>

#ifdef HAVE_STDINT_H
#include <stdint.h>
#endif
#ifdef HAVE_INTTYPES_H
#include <inttypes.h>
#endif

#include "macaddr.h"
#include "ieee80211.h"

// Like a config file
class HostapStatFile {
public:
    int ParseStats(const char *in_fname);
    unsigned long int FetchStat(string in_key);

    char *FetchError() {
        return errstr;
    }
    
protected:
    map<string, unsigned long int> stat_map;

    char errstr[1024];
};

// Relevant info about each station
typedef struct {
    mac_addr sta_mac;
    unsigned int users; // What is this?
    unsigned int aid;
    unsigned int rxpackets;
    unsigned int txpackets;
    unsigned long int rxbytes;
    unsigned long int txbytes;
} hostap_station_rec;

// Find all the station records in a directory.  It's safe to use a map<macaddr> 
// instead of a macmap because these will never be masked
int HostAPFetchStations(const char *in_hostdir, char *errstr,
                        map<mac_addr, hostap_station_rec> *ret_map);

// desired MTU
#define HOSTAP_RAWTX_MTU 2290

// Create a raw socket into hostap
int HostAPCreateRawSock(const char *in_if, char *errstr);

// Write a frame to a raw socket
int HostAPWriteFrame(int in_sock, uint8_t *in_xmit, int in_len, char *errstr);

#endif

