/*
    This file is part of Kismet

    Kismet is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Kismet is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Kismet; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __TCPMONITORSERVER_H__
#define __TCPMONITORSERVER_H__

#include "config.h"

#include <time.h>

#include "tcpserver.h"
#include "ap_monitor_proto.h"
#include "tcpsslapserver.h"

// Simple push-server to serialize the state of the controller and send it to
// something else for visualization

int MonitorPushTimer(Timetracker::timer_event *evt, void *parm);

class TcpMonitorMessageClient;

class TcpMonitorServer : public TcpServer {
public:
    typedef struct {
        // Crummy little maps to define when we last pushed something to this
        // client.  Theres probably a better way to do this, but what the hell.
        map<TcpSSLApServer::apserver_apinterface *, time_t> if_sent_map;
        map<TcpSSLApServer::apserver_aprec *, time_t> ap_sent_map;
        map<int, time_t> zone_sent_map;
        int dead;
        int pristine;
        unsigned long int seqid;
    } client_info;

    TcpMonitorServer();
    virtual ~TcpMonitorServer();

    // Learn the ap server we'll be querying
    virtual void RegisterApServer(TcpSSLApServer *in_apserver);

    // Hook the setup to start our timer
    virtual int Setup(unsigned int in_max_clients, short int in_port);

    // Kill a con
    virtual void KillConnection(int in_fd);

    // Push text message to all clients
    virtual int TcpMonitorServer::PushMessageAll(const char *in_msg, int in_msgflag);

protected:
    // bitbucket the incoming data, but we need to do something
    virtual int ParseData(int in_fd);

    // Hook the acceptance
    virtual int Validate(int in_fd);

    // Trigger the timer event
    virtual void PushTimer();

    // Push various record types
    virtual int PushInfoRecord(int in_fd);
    virtual int PushZoneRecord(int in_fd, TcpSSLApServer::apserver_zonerec *in_zone);
    virtual int PushApRecord(int in_fd, TcpSSLApServer::apserver_aprec *in_ap);
    virtual int PushIfRecord(int in_fd, TcpSSLApServer::apserver_apinterface *in_if);
    virtual int PushMessageRecord(int in_fd, const char *in_msg, int in_msgflag);

    int timer_id;

    int QueuePacket(int in_fd, monitor_frame *in_frame);

    TcpSSLApServer *apsslserver;

    map<int, TcpMonitorServer::client_info *> monitor_recs;

    friend int MonitorPushTimer(Timetracker::timer_event *evt, void *parm);
    friend class TcpMonitorMessageClient;
};

class TcpMonitorMessageClient : public MessageClient {
public:
    void ProcessMessage(string in_msg, int in_flags);
    void RegisterMonitorServer(TcpMonitorServer *in_srv) {
        srv = in_srv;
    }
protected:
    TcpMonitorServer *srv;
};

#endif

